#include <stdlib.h>
#include <stdint.h>

#include <SFML/System.h>
#include <SFML/Graphics.h>
#include <SFML/Window.h>

/* Public Variables */
extern sfVertexArray *renderArray;
int8_t videoModified;

/* Hardware-like Registers */
uint8_t videoX;
uint8_t videoY;
uint8_t videoMode;
uint8_t videoA;

/* Private Variables */
static uint8_t videoRow;
static uint8_t videoValue;
static uint8_t videoColorValue;
static sfColor videoDisplayColor;
static sfColor videoForeColor;
static sfColor videoBackColor;

const static sfColor videoUpperPalette = (sfColor){ 64, 64, 64 };

static sfColor *videoColorIndex[] = { 
	&sfBlack, &sfRed, &sfGreen, &sfYellow, &sfBlue, &sfMagenta, &sfCyan, &sfWhite//&(sfColor){ 96, 96, 96 } ,
	/*&(sfColor){ 0, 0, 0 },
	&(sfColor){ 0xFF, 0, 0 },
	&(sfColor){ 0, 0xFF, 0 },
	&(sfColor){ 0xFF, 0xFF, 0 },
	&(sfColor){ 0, 0, 0xFF },
	&(sfColor){ 0xFF, 0, 0xFF },
	&(sfColor){ 0, 0xFF, 0xFF },
	&(sfColor){ 0xFF, 0xFF, 0xFF },*/
};
static uint8_t videoMemory[0x8000];


void writeVideo(){
	videoMemory[videoX + (videoY * 128)] = videoA;
	videoModified = 1;
}

uint8_t readVideo(){
	return videoMemory[videoX + (videoY * 128)];
}

void initVideo(){
	renderArray = sfVertexArray_create();
	sfVertexArray_setPrimitiveType(renderArray, sfQuads);
	
	videoModified = 1;
	videoMode = 0x01;
	
	for (int i = 0; i < 0x8000; i++) videoMemory[i] = rand();
}

void updateVideo(){
	if (videoModified){
		sfVertexArray_clear(renderArray);
		
		for (uint8_t y = 0; y < 255; y++){
			videoRow = (int8_t)(y & 0x7);
			
			for (uint8_t x = 0; x < 48; x++){
				
				if ((videoMode & 0x01) == 0){
					/*  
					*           Text Address Format
					*           0bRRR1111AAAAAAAA
					* 
					*           R = row pins
					*           A = address
					*/
					
					// Get text index value
					videoValue = videoMemory[(((y & 0xF8) >> 3) << 7) | x];
					
					// Index the value in font location
					videoValue = videoMemory[(videoRow | 0b1111000) + (videoValue * 128)];
					videoColorValue = videoMemory[(((y & 0xF8) >> 3) << 7) + (x | 0x40)];
				}
				else {
					videoValue = videoMemory[(y * 128) + x];
					videoColorValue = videoMemory[(y * 128) + (x | 0x40)];
				}
				
				// Snipping out the upper 8 colors
				videoColorValue &= 0x77;
				
				videoForeColor = *videoColorIndex[(videoColorValue & 0x0F)];
				videoBackColor = *videoColorIndex[((videoColorValue & 0xF0) >> 4)];
				
				
				for (uint8_t i = 0; i < 8; i++){
					if (videoValue & (128 >> i)) videoDisplayColor = videoBackColor;
					else videoDisplayColor = videoForeColor;
					
					sfVertexArray_append(renderArray, (sfVertex){ (sfVector2f){ (x * 8) + i, y }, videoDisplayColor});
					sfVertexArray_append(renderArray, (sfVertex){ (sfVector2f){ (x * 8) + i + 1, y }, videoDisplayColor});
					sfVertexArray_append(renderArray, (sfVertex){ (sfVector2f){ (x * 8) + i + 1, y + 1 }, videoDisplayColor});
					sfVertexArray_append(renderArray, (sfVertex){ (sfVector2f){ (x * 8) + i, y + 1 }, videoDisplayColor});
				}
			}
		}
		
		videoModified = 0;
	}
}



